@@ -12,7 +12,7 @@
 //! # Example
 //!
 //! ```no_run
-//! use poem::{get, handler, listener::TcpListener, web::Path, Route, Server};
+//! use poem::{get, handler, listener::TcpListener, web::Path, IntoResponse, Route, Server};
 //!
 //! #[handler]
 //! fn hello(Path(name): Path<String>) -> String {
@@ -42,6 +42,8 @@
 //! |tls               | Support for HTTP server over TLS   |
 //! |tempfile          | Support for [`tempfile`](https://crates.io/crates/tempfile) |
 //! |tower-compat      | Adapters for `tower::Layer` and `tower::Service`. |
+//! |session           | Support for CookieSession    |
+//! |redis-session     | Support for RedisSession     |
 
 #![doc(html_favicon_url = "https://poem.rs/assets/favicon.ico")]
 #![doc(html_logo_url = "https://poem.rs/assets/logo.png")]
@@ -55,6 +57,9 @@ pub mod error;
 pub mod listener;
 pub mod middleware;
 pub mod service;
+#[cfg(feature = "session")]
+#[cfg_attr(docsrs, doc(cfg(feature = "session")))]
+pub mod session;
 pub mod web;
 
 #[doc(inline)]
@@ -74,6 +79,8 @@ pub use middleware::Middleware;
 pub use poem_derive::handler;
 pub use request::{Request, RequestBuilder, RequestParts};
 pub use response::{Response, ResponseBuilder, ResponseParts};
-pub use route::{connect, delete, get, head, options, patch, post, put, trace, Route, RouteMethod};
-pub use server::{warps_endpoint, Server};
+pub use route::{
+    connect, delete, get, head, options, patch, post, put, trace, Route, RouteDomain, RouteMethod,
+};
+pub use server::Server;
 pub use web::{FromRequest, IntoResponse, RequestBody};
