@@ -48,7 +48,7 @@ pub use typed_header::TypedHeader;
 pub use self::tempfile::TempFile;
 use crate::{
     body::Body,
-    error::{Error, ReadBodyError, Result},
+    error::{ReadBodyError, Result},
     http::{
         header::{HeaderMap, HeaderName},
         HeaderValue, Method, StatusCode, Uri, Version,
@@ -262,8 +262,9 @@ impl RequestBody {
 pub trait FromRequest<'a>: Sized {
     /// The error type of this extractor.
     ///
-    /// If you don't know what type you should use, you can use [`Error`].
-    type Error: Into<Error>;
+    /// If you don't know what type you should use, you can use
+    /// [`Error`](crate::Error).
+    type Error: IntoResponse;
 
     /// Perform the extraction.
     async fn from_request(req: &'a Request, body: &mut RequestBody) -> Result<Self, Self::Error>;
@@ -313,6 +314,10 @@ pub trait FromRequest<'a>: Sized {
 /// `application/octet-stream`. The vector’s data is used as the body of the
 /// response.
 ///
+/// - **Body**
+///
+///     Sets the status to `OK` and use the specified body.
+///
 /// - **StatusCode**
 ///
 ///    Sets the status to the specified status code [`StatusCode`] with an empty
@@ -575,9 +580,9 @@ impl IntoResponse for () {
     }
 }
 
-impl IntoResponse for Infallible {
+impl IntoResponse for Body {
     fn into_response(self) -> Response {
-        Response::builder().body(Body::empty())
+        Response::builder().body(self)
     }
 }
 
@@ -612,7 +617,7 @@ impl<T: IntoResponse> IntoResponse for (HeaderMap, T) {
     }
 }
 
-impl<T, E> IntoResponse for core::result::Result<T, E>
+impl<T, E> IntoResponse for std::result::Result<T, E>
 where
     T: IntoResponse,
     E: IntoResponse,
@@ -749,6 +754,7 @@ impl<'a, T: FromRequest<'a>> FromRequest<'a> for Result<T, T::Error> {
 #[cfg(test)]
 mod tests {
     use super::*;
+    use crate::Error;
 
     #[tokio::test]
     async fn into_response() {
